import { readdir, writeFile, mkdir, readFile } from "fs/promises";
import { Command } from "commander";
import inquirer from "inquirer";

type BlogMeta = {
  name: string;
  description: string;
  id: number;
  created_at: number;
  updated_at: number;
};

const EDITOR = process.env["EDITOR"];
const BLOGS_DIR = process.env["BLOGS_DIR"] ?? "/blogs";

export function app() {
  const program = new Command();
  const blog_app = program.command("blog");

  blog_app.command("create").action(async () => {
    await create_blog();
  });

  blog_app
    .command("edit", "command to edit blog")
    .argument("<id>")
    .action(async (id) => {
      await edit_blog(id);
    });

  blog_app.parse();
}

async function create_blog() {
  if (!EDITOR) {
    process.env["EDITOR"] = "nvim";
  }

  const user_blog_meta = await inquirer.prompt<UserBlogMeta>([
    {
      name: "name",
      message: "What is the title of your blog?",
    },
    {
      name: "description",
      message: "Enter a short description for your blog",
    },
    {
      type: "editor",
      name: "content",
      message: "Please enter your blog now.",
    },
  ]);

  const created_at = Date.now();
  const updated_at = created_at;
  const blog_id = await get_new_blog_id();
  const { name, description, content } = user_blog_meta;

  const new_blog_meta: BlogMeta = {
    name,
    description,
    id: blog_id,
    created_at,
    updated_at,
  };

  await save_blog(new_blog_meta, content);
}

async function edit_blog(blog_id: string) {
  if (!EDITOR) {
    process.env["EDITOR"] = "nvim";
  }
  const blog_path = `${BLOGS_DIR}/blog_${blog_id}`;

  const [blog_content, blog_meta] = await Promise.all([
    get_blog(blog_path),
    get_blog_meta(blog_path),
  ]);

  const { content } = await inquirer.prompt<{ content: string }>([
    {
      type: "editor",
      name: "content",
      default: blog_content,
      message: "Please enter your blog now.",
    },
  ]);

  const new_blog_meta = { ...blog_meta, updated_at: Date.now() };

  await save_blog(new_blog_meta, content);
}

async function get_new_blog_id() {
  const blog_count = (await readdir(BLOGS_DIR, { withFileTypes: true })).filter(
    (dirent) => dirent.isDirectory()
  ).length;

  return blog_count + 1;
}

async function save_blog(blog_meta: BlogMeta, content: string) {
  const path = `${BLOGS_DIR}/blog_${blog_meta.id}`;

  await mkdir(path, { recursive: true });

  await Promise.all([
    write_meta_data(blog_meta, path),
    write_blog(content, path),
  ]);
}

async function get_blog(path: string) {
  return (await readFile(path + "/blog.md", "utf8")).toString();
}

async function get_blog_meta(path: string) {
  const meta: BlogMeta = JSON.parse(
    (await readFile(path + "/meta.json", "utf8")).toString()
  );

  return meta;
}

async function write_meta_data(blog_meta: BlogMeta, path: string) {
  const data = JSON.stringify(blog_meta);

  await writeFile(path + "/meta.json", data);
}

async function write_blog(data: string, path: string) {
  await writeFile(path + "/blog.md", data);
}

type UserBlogMeta = Pick<BlogMeta, "name" | "description"> & {
  content: string;
};
